/*
 * main.c
 *
 *  Created on: 22.11.2017
 *      Author: rieche
 */

#include "hal/hal.h"
#include "riscv_hal/riscv_hal.h"
#include "riscv_hal/sample_hw_platform.h"
#include "drivers/CoreUARTapb/core_uart_apb.h"
#include "drivers/CoreGPIO/core_gpio.h"

#define DIVISOR 256

/******************************************************************************
 * GPIO instance data. UART instance data. Base addresses
 *****************************************************************************/
gpio_instance_t g_gpio_out, g_gpio_in;
UART_instance_t g_uart0, g_uart1;

/******************************************************************************
 * semaphores for event handling
 *****************************************************************************/
int sema_gpio = 0;

/******************************************************************************
 * function prototypes
 *****************************************************************************/
void LED_update(void);

/******************************************************************************
 * interrupt service routines
 *****************************************************************************/
void External_2_IRQHandler(void)
{
	sema_gpio = 1;
	GPIO_clear_all_irq_sources(&g_gpio_in, GPIO_get_irq_sources(&g_gpio_in));
}

/******************************************************************************
 * MAIN
 *****************************************************************************/
int main()
{
	PLIC_init();

	UART_init(
			&g_uart0,
			COREUARTAPB0_BASE_ADDR,
			BAUD_VALUE_115200,
			(DATA_8_BITS | NO_PARITY)
	);
	UART_init(
			&g_uart1,
			COREUARTAPB1_BASE_ADDR,
			BAUD_VALUE_115200,
			(DATA_8_BITS | NO_PARITY)
	);


    /**************************************************************************
     * Initialize the CoreGPIO driver with the base address of the CoreGPIO
     * instance to use and the initial state of the outputs.
     *************************************************************************/
	/* OUTPUTS */
	GPIO_init(&g_gpio_out, COREGPIO_OUT_BASE_ADDR, GPIO_APB_32_BITS_BUS );

	/* INPUTS */
	GPIO_init(&g_gpio_in, COREGPIO_IN_BASE_ADDR, GPIO_APB_32_BITS_BUS );

	/* In this version of the PLIC, the priorities are fixed at 1.
	Lower numbered devices have higher priorities. But this code is given as
	an example.
	*/
	PLIC_SetPriority(External_2_IRQn, 1);
	PLIC_EnableIRQ(External_2_IRQn);

	GPIO_enable_irq(&g_gpio_in, GPIO_0);
	GPIO_enable_irq(&g_gpio_in, GPIO_1);
	GPIO_enable_irq(&g_gpio_in, GPIO_2);
	GPIO_enable_irq(&g_gpio_in, GPIO_3);

	HAL_enable_interrupts();

	UART_polled_tx_string(&g_uart0, (uint8_t *)"UART0, Hello World...\n\r");
	UART_polled_tx_string(&g_uart1, (uint8_t *)"UART1, Hello World...\n\r");

	while(1) {
		static uint32_t counter = 0;
		uint8_t key;

    	/* uart0 rx -> uart1 tx */
		if(UART_get_rx(&g_uart0, &key, 1)) {
			UART_send(&g_uart1, &key, 1);
		}
		/* uart1 rx -> uart0 tx */
		if(UART_get_rx(&g_uart1, &key, 1)) {
			UART_send(&g_uart0, &key, 1);
		}
		/* push button hit -> counter reset */
		if(sema_gpio) {
			counter = 0;
			sema_gpio = 0;
		}
		/* counter update */
		if(counter-- ==  0) {
			counter = (SYS_CLK_FREQ/DIVISOR);
			LED_update();
		}
	}
	return 0;
}

/******************************************************************************
 * function
 *****************************************************************************/
void LED_update(void)
{
	static uint8_t temp = 1;

	temp = temp^1;
	GPIO_set_output(&g_gpio_out, GPIO_0, temp);
	GPIO_set_output(&g_gpio_out, GPIO_1, !temp);
	GPIO_set_output(&g_gpio_out, GPIO_2, temp);
	GPIO_set_output(&g_gpio_out, GPIO_3, !temp);
}
